/*
Copyright (c) 2001, John Lambert jlambert@jlambert.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are 
permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of 
conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other
materials provided with the distribution. 

Neither the name of the xdProf project nor the names of its contributors may be used
to endorse or promote products derived from this software without specific prior 
written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
$Header: /xdprof-dll/classes.cpp 19    5/09/01 12:58a Admin $ 
 */
/** \file classes.cpp
 * Things related to class loading and unloading.
 */

#include "xdprof.h"

/**
Unloads a class
@param class_id the class to unload
*/
void ClassUnloadEvent(jobjectID class_id)
{
	Lock(methods_lock);
	ClassMethopMap::iterator i;
	i = ClassMethods.find(class_id);
	
	const ClassInfoMap::iterator & ci = Classes.find(class_id);
	if (ci != Classes.end())
		debug("Unload ", class_id, ci->second.getShortInfo());
	else
		debug("Unload ", class_id);
	
	if (i != ClassMethods.end())
	{
		MethodList & ms = i->second;

		debug("erase before ", Methods.size(), MethodCache.size());
		MethodList::iterator m;
		for (m = ms.begin(); m != ms.end(); ++m)
		{
			const MethodInfo* mi = Methods[*m];
			Methods.erase(*m);
			MethodCache.erase(*m);
			delete mi;			
		}
		debug("erase after ", Methods.size(), MethodCache.size());
	}
	
	ClassMethods.erase(class_id);
	Classes.erase(class_id);
	ClassCache.erase(class_id);
	
	Unlock(methods_lock);
}

#pragma warning(disable: 4100)
/**
 Called upon class load.
 */
void ClassLoadEvent(JNIEnv *env_id, 
			    char** class_name,
			    char** source_name,
			    int num_interfaces,
			    int num_statics,
			    JVMPI_Field *statics,
			    int num_instances,
			    JVMPI_Field *instances,
			    int num_methods,
			    JVMPI_Method *methods,
			    jobjectID class_id,
				jint eventType)
{
	//Lock(methods_lock);
	if (ClassMethods.find(class_id) != ClassMethods.end()) // then it exists
	{
		debug("!!! exists!!!", class_id, *class_name);
	}
	
	debug((eventType == (JVMPI_EVENT_CLASS_LOAD | JVMPI_REQUESTED_EVENT)) ? "Requested load" : "load", (*class_name) , *source_name == null ? "unknown" : *source_name , class_id);

	//debug("count", *class_name, num_instances , num_statics);

	// load method table
	JVMPI_Method* jm = methods;
	//string className(class_name);
	MethodList & temp_methods = ClassMethods[class_id]  ;
	//string sourceName;
	/*
	if (source_name != null)
		sourceName = string(source_name);
	else
		sourceName = "unknown";
	*/

	for (int i = 0; i < num_methods; i++)
	{
		debug("method", jm[i].method_id, jm[i].method_name, jm[i].method_signature);//, jm[i].start_lineno, jm[i].end_lineno, jm[i].method_id, 
		//const MethodInfo *mi = new MethodInfo(jm[i].method_name, jm[i].method_signature, jm[i].start_lineno, jm[i].end_lineno, jm[i].method_id, class_id);
		//const pair<jmethodID, const MethodInfo*> p(jm[i].method_id, mi);
		//Methods.insert(p);
		Methods.insert(make_pair(jm[i].method_id, new MethodInfo(jm[i].method_name, jm[i].method_signature, jm[i].start_lineno, jm[i].end_lineno, jm[i].method_id, class_id)));
		//Methods[jm[i].method_id] = new MethodInfo(jm[i].method_name, jm[i].method_signature, jm[i].start_lineno, jm[i].end_lineno, jm[i].method_id, class_id);

		temp_methods.push_back(jm[i].method_id);
	}

	//ClassMethods[class_id] = temp_methods;
	
	//ClassInfo ci(*class_name, *source_name, class_id);
	//assert(false);
	Classes.insert(make_pair(class_id, ClassInfo(*class_name, *source_name, class_id) ));
	//Unlock(methods_lock);
}
