package org.xdprof.data;

import org.xdprof.util.*;

/**
 * Base class for Identifiers like MethodIDs and ClassIDs and ThreadIDs
 */
abstract public class Identifier implements Comparable
{
	/**
	 * Fast value.
	 */
	private final Long m_value;
	
	/**
	 * Original in Hex.
	 */
	private final String m_original;
	
	public String toString()
	{
		//Assert.assert(m_value.toString(m_value.longValue(), 16) + " " + m_original, m_value.toString(m_value.longValue(), 16).equals(m_original));
		return m_original;	
	}
	
	public Identifier()
	{
		Assert.failNotImplemented();
		m_value = null;
		m_original = null;
	}
	
	/**
	 * a negative integer, zero, or a positive integer as this 
	 * object is less than, equal to, or greater than the
	 * specified object.
	 */
	public int compareTo(Identifier i)
	{
		return m_value.compareTo(i.m_value);	
	}
	
	public int compareTo(Object o)
	{
		Assert.assert(o.getClass().toString(), o.getClass().equals( this.getClass() ) );
		return compareTo( (Identifier)o);
	}
	
	public boolean equals(Object o)
	{
		Assert.assert(o.getClass().toString() + " " + this.getClass().toString(), o.getClass().equals( this.getClass() ) );
		return m_value.equals( ((Identifier)o).m_value);
	}
	
	public Identifier(String v)
	{
		// TODO: attempt to find/lookup?
		if(v.startsWith("0x") || v.startsWith("0X"))
			m_original = v.substring(2).toUpperCase();
		else
			m_original = v.toUpperCase();
		try
		{
			m_value = new Long(Long.parseLong(m_original, 16));
		}
		catch (NumberFormatException nfe)
		{
			throw nfe;
		}
	}
	
	public int hashCode()
	{
		return m_value.hashCode();	
	}
}
