package org.xdprof.data;

import org.xdprof.util.*;

public class MethodInfo
{
	private final ClassIdentifier m_classID;
	private final MethodIdentifier m_methodID;
	private final String m_name, m_signature;
	private final String m_resolved;
	
	public MethodInfo(ClassIdentifier cid, MethodIdentifier mid, String name, String signature)
	{
		m_classID = cid;
		m_methodID = mid;
		m_name = name;
		m_signature = signature;
		m_resolved = fixIt(m_name, m_signature);
	}
	
	public ClassIdentifier getClassID()
	{
		return m_classID;	
	}
	
	public MethodIdentifier getMethodID()
	{	
		return m_methodID;
	}
	
	public String getFullName()
	{
		return m_resolved;
	}
	
	/**
	 * Converts a full compacted method declaration to a human readable form.
	 * @param typeSig the type signature
	 * @param methodName the method name
	 */
	private static final String fixIt(String methodName, String typeSig)
	{
		String ret;
		//Debug.trace("typeSig", typeSig);
		// get return type
		String returnType = fixType(typeSig.substring(typeSig.lastIndexOf(")") + 1));
		//Debug.trace("retType: ", returnType);
		ret = returnType + " " + methodName;

		String methodParams = typeSig.substring(1, typeSig.lastIndexOf(")") );
		//Debug.trace("methodParams ", methodParams);

		if (methodName.equals(""))
			return ret + "()";
		
		ret = ret + "(";
		final String nativeTypes = "ZBCSIJFDV";
		if (methodParams.equals(""))
			return ret + ")";
		
		while (methodParams.length() > 0)
		{
			int i;
			//Debug.trace(" methodParams",  methodParams);
			if (methodParams.charAt(0) != 'L' &&
				nativeTypes.indexOf(methodParams.charAt(0)) != -1) // it's a	native type
				i = 1;
			else
				if (methodParams.charAt(0) == 'L') // it's a class
					i = methodParams.indexOf(";") + 1;
				else // it's an array
				{
					// [[Ljava/lang/String;
					int j = methodParams.indexOf("[");
					while (methodParams.charAt(j) == '[')
						j++;
					if (methodParams.charAt(j) == 'L')
						j = methodParams.indexOf(";", j);
					//Debug.trace("j " + j, methodParams.charAt(j) );
					i = j + 1;
				}
			//Debug.trace( "i " ,  i );
			//Debug.trace( "mPsub " , methodParams.substring(0, i)  );
			ret = ret + fixType(methodParams.substring(0, i)) + ", ";
			methodParams = methodParams.substring(i);
		}
		ret = ret.substring(0, ret.length() - 2);
		ret = ret + ")";
		return ret;
	}
	/**
	 * Converts Java type strings ([Z) to human-readable form (boolean[]).
	 * @param type the type name
	 */
	private static final String fixType(String type)
	{
		if (type.charAt(0) == '[')
			return fixType(type.substring(1)) + "[]";
		
		char c = type.charAt(0);
		switch (c)
		{
		case 'L':
			String t = type.substring(1, type.length() - 1);
			return t.replace('/', '.');
		case 'Z': return "boolean";
		case 'B': return "byte";
		case 'C': return "char";
		case 'S': return "short";
		case 'I': return "int";
		case 'J': return "long";
		case 'F': return "float";
		case 'D': return "double";
		case 'V': return "void";
		}
		Assert.failNotReachable(type );
		return "";
	}
	
	public String toString()
	{
		return m_resolved;
	}
}
