package org.xdprof.data;

import java.util.*;
import org.xdprof.util.*;

/**
 * This represents an instaneous trace of a connection; 
 * it contains threads, classes, methods, and call stacks.
 */
public class Trace
{
	/**
	 * When this trace happened.
	 */
	private final Timestamp m_when;
	
	/**
	 * When this 
	 */
	private final String m_info;
	
	private static int m_c = 0;
	private int m_mine;
	int m_pid;
	String m_description;
	String m_os;
	String m_rt;
	String m_vm;
	HashMap m_properties = new HashMap();

	/**
	 * Constructs a new trace with a timestamp.
	 */
	public Trace(String info)
	{
		m_mine = ++m_c;
		m_info = info;
		String s = info;
		
		int i = s.indexOf(' ');
		m_pid = Integer.parseInt(s.substring(0, i));
		m_description = s.substring(i + 1, s.indexOf('!',i) - 1);
		StringTokenizer st = new StringTokenizer(s.substring(s.indexOf('!',i)), "!");
		while (st.hasMoreTokens())
		{
			String p = st.nextToken();
			m_properties.put(p.substring(0, p.indexOf('=')), p.substring(p.indexOf('=')+1));
		}
		
//		Debug.trace(m_properties.keySet().toString());
//		Debug.trace(m_properties.values().toString());
		
		m_os = (m_properties.get("os.name") + " " + m_properties.get("os.version") + " " + m_properties.get("os.arch")).trim();
		m_vm = (m_properties.get("java.vm.name") + " " + 
			   m_properties.get("java.vm.version") + " "  + 
			   m_properties.get("java.vm.info") + " "  + 
			   m_properties.get("java.vm.vendor")).trim();
		
		m_rt = (m_properties.get("java.runtime.name") + " "  + 
			   m_properties.get("java.runtime.version")).trim();
		//Debug.trace("os", m_os);		Debug.trace("vm", m_vm);		Debug.trace("rt", m_rt);
		
		m_when = new Timestamp();	
	}
	
	public String toString()
	{
		return m_threads.toString();
		//return "trace" + m_mine;	
	}
		
	/**
	 * Map<classID, ClassInfo>; classes in use.
	 */
	private final Map m_classesInUse = new HashMap();
	
	/**
	 * Map<methodID, methodInfo>; methods in use.
	 */
	private final Map m_methodMap = new TreeMap();
	
	/**
	 * Map<classID, Set<MethodID> >
	 */
	private final Map m_classIDToMethodIDs = new HashMap();
	
	/**
	 * This adds a value to a set contained in map(key); uses HashSet.
	 * @param m a map from key to Set(value)
	 * @param key the key to use when looking through the map
	 * @param value the value to put in the set
	 */
	private static void addItemToSet(Map m, Object key, Object value)
	{
		Assert.notNull(m);
		Assert.notNull(key);
		Assert.notNull(value);

		Set s = (Set) m.get(key);
		if (s == null)
		{
			s = new HashSet();
			m.put(key, s);
		}
		s.add(value);
	}
	
	/**
	 * Returns a set of MethodIdentifiers for the class.
	 * @param classID
	 * @return a set of MethodIdentifiers
	 */
	public Set getMethodIDsForClass(ClassIdentifier classID)
	{
		Assert.assert("there", m_classesInUse.containsKey(classID));
		return (Set)m_classIDToMethodIDs.get(classID);
	}
	
	/**
	 * Returns true if a thread is in here.
	 */
	public boolean containsThread(ThreadIdentifier tid)
	{
		return m_threadIDs.contains(tid);
	}
	
	/**
	 * List<ThreadInfo>
	 */
	private final List m_threads = new LinkedList();
	
	/**
	 * Set<ThreadIdentifiers>
	 */
	private final Set m_threadIDs = new HashSet();
	
	/**
	 * Add a thread and return a reference to it.
	 * @param threadID the thread ID
	 * @param name the thread name
	 * @param group the thread group
	 * @param parent the thread parent
	 * @param started when this thread was started
	 * @param status the thread status
	 * @return a reference to the added thread.
	 */
	public ThreadInfo addThread(ThreadIdentifier threadID, String name, String group, String parent, int started, int status)
	{
		ThreadInfo ti = new ThreadInfo(this, threadID, name, group, parent, started, status);
		m_threads.add(ti);
		m_threadIDs.add(threadID);
		return ti;
	}
	
	/**
	 * Finds the MethodInfo for a method id (or null if it's not there).
	 * @param methodID the method identifier
	 */
	public MethodInfo findMethodInfo(MethodIdentifier methodID)
	{
		/*
		if (!m_methodMap.containsKey(methodID))
		{
			Debug.trace(toString() + " not find " + methodID.toString() + " in " +m_methodMap.keySet());	
			System.exit(-1);
		}
		else
			Debug.trace(toString() + " " + m_methodMap.keySet().toString());
		*/
		Assert.assert("there", m_methodMap.containsKey(methodID));
		return (MethodInfo) m_methodMap.get(methodID);
	}
	
	/**
	 * Adds a method and returns a reference to it.
	 * @param classID the class ID
	 * @param methodID the method ID
	 * @param methodName the name of the method
	 * @param methodSignature the java--based name of the method
	 */
	public MethodInfo addMethod(ClassIdentifier classID, MethodIdentifier methodID, String methodName, String methodSignature)
	{
		Assert.assert("not there", ! m_methodMap.containsKey(methodID));
		//Debug.trace(toString() +  " adding method " + methodID);
		
		MethodInfo mi = new MethodInfo(classID, methodID, methodName, methodSignature);
		m_methodMap.put(methodID, mi);
		addItemToSet(m_classIDToMethodIDs, classID, methodID);
		return mi;
	}
	
	/**
	 * Returns a Vector of ClassInfo.
	 */
	public Vector getClasses()
	{
		return new Vector(m_classesInUse.values());
	}
	
	public void selfCheck()
	{
		for (Iterator i = m_threads.iterator(); i.hasNext(); )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			for (int k = 0; k < ti.getFrameCount(); ++k)
			{
				if (!m_methodMap.containsKey(ti.getFrameMethod(k)))
					System.err.println("!!!!");
//				System.err.println(ti.getFrameMethod(k) + " " + " is there ");
				Assert.assert(ti.getFrameMethod(k).toString(), m_methodMap.containsKey(ti.getFrameMethod(k)));
			}
		}
	}
	
	/**
	 * Returns a Vector of ThreadInfo
	 */
	public Vector getThreads()
	{
		Vector v = new Vector(m_threads);
		Collections.sort(v);
		return v;	
	}
	
	
	/**
	 * 
	 */
	public ThreadInfo getThreadInfo(ThreadIdentifier ti)
	{
		for (ListIterator li = m_threads.listIterator(); li.hasNext(); )
		{
			ThreadInfo inf = (ThreadInfo)li.next();
			if (inf.getThreadID().equals(ti))
				return inf;
		}
		
		Assert.failNotReachable();
		return null;
	}
	
	/**
	 * Returns a ClassInfo block for a classID.
	 * @param classID the class ID
	 */
	public ClassInfo findClassInfo(ClassIdentifier classID)
	{
		Assert.assert("there", m_classesInUse.containsKey(classID));
		return (ClassInfo) m_classesInUse.get(classID);
	}
	
	/**
	 * Returns the ClassInfo for a MethodIdentifier.
	 * @param mid the method id
	 */
	public ClassInfo findClassInfo(MethodIdentifier mid)
	{
		return findClassInfo(findMethodInfo(mid).getClassID());
	}
	
	/**
	 * Adds a class and returns a reference to it.
	 * @param classID the class ID
	 * @param className the name of the class (java/lang/String)
	 * @param classSource the source file of the class
	 */
	public ClassInfo addClass(ClassIdentifier classID, String className, String classSource)
	{
		Assert.assert("not there", ! m_classesInUse.containsKey(classID));
		ClassInfo ci = new ClassInfo(classID, className, classSource);
		m_classesInUse.put(classID, ci);
		return ci;
	}
}
