package org.xdprof.listener;

import java.io.*;
import java.net.*;
import java.util.*;

import org.xdprof.data.*;
import org.xdprof.server.*;
import org.xdprof.util.*;

/**
 * A console-based server.
 */
public class CodeCoverageListener implements ServerListener
{
	private Set m_removed = new HashSet();
	
	/**
	 * This adds a value to a set contained in map(key); uses HashSet.
	 * @param m a map from key to Set(value)
	 * @param key the key to use when looking through the map
	 * @param value the value to put in the set
	 */
	private static void addItemToSet(Map m, Object key, Object value)
	{
		Assert.notNull(m);
		Assert.notNull(key);
		Assert.notNull(value);

		Set s = (Set) m.get(key);
		if (s == null)
		{
			s = new HashSet();
			m.put(key, s);
		}
		s.add(value);
	}

	
	/**
	 * Construct on this port.
	 */
	public CodeCoverageListener()
	{
		//new Server(port, this).start();
	}
	
	public void setPort(int p)
	{
		// do nothing
	}
	
	/**
	 * Removes/unregisters a connection.
	 * @param c the connection to remove
	 */
	public void removeConnection(Connection c)
	{
		Debug.trace("TEXT: Remove", c.toString());
		m_removed.add(c);
				
		Iterator allmeth = m_allInfo.keySet().iterator();
		while (allmeth.hasNext())
		{
			MethodIdentifier mid = (MethodIdentifier)allmeth.next();
			Set s = (Set)m_methods.get(mid); // s is line numbers
			//System.out.println(mid + " " + s.toString());
			Integer m = (Integer)Collections.max(s);
			int mval = m.intValue();
			int hit = 0;
			for (int i = 0; i <= mval; i++)
				if (s.contains(new Integer(i)))	++hit;
			//Debug.trace(mid.toString() + t.toString());
			System.out.println(((MethodInfo)m_allInfo.get(mid)).getFullName() + " " + s.toString() + " " + hit + "/" + mval);
		}
	}
	
	/**
	 * Adds a connection to the server.
	 * @param c the connection to register
	 */
	public void addConnection(Connection c)
	{
		Debug.trace("TEXT: Add connection", c);
	}

	
	private Map m_allInfo = new HashMap();
		Map m_methods = new HashMap();	
	/**
	 * Adds a trace for the connection.
	 * @param c the connection
	 * @param t the "latest" trace
	 */
	public void addTrace(Connection c, final Trace t)
	{

		Assert.assert("Not shut down", !m_removed.contains(c));
		//Debug.trace("TEXT: Adding trace for " + c + " " + t.toString());
		Vector v = t.getThreads();
		for (Iterator i = v.iterator(); i.hasNext(); )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			
			for (int meth = 0; meth < ti.getFrameCount(); meth++)
			{
				addItemToSet(m_methods, ti.getFrameMethod(meth), new Integer(ti.getFrameLineNumber(meth)));
				if (!m_allInfo.containsKey(ti.getFrameMethod(meth)))
					m_allInfo.put(ti.getFrameMethod(meth), t.findMethodInfo(ti.getFrameMethod(meth)));
			}
		}

	}
		
	/**
	 * Start the program.
	 * @param args args[0] is the port number
	 */
	public static void main(String[] args)
	{
		try
		{
			Server.startServer(Integer.parseInt(args[0]), new CodeCoverageListener());
		}
		catch (Exception e)
		{
			Debug.traceException("main", e);
			System.err.println("Usage: ServerText port");
		}
	}

}
