package org.xdprof.listener;

import org.xdprof.data.*;
import org.xdprof.server.*;
import org.xdprof.util.*;

import java.util.*;
import java.io.*;

/**
  digraph structs {
node [shape=record];
    struct1 [shape=record,label="<f0> left|<f1> middle|<f2> right"];
    struct2 [shape=record,label="<f0> one|<f1> two"];
    struct3 [shape=record,label="hello\nworld |{ b |{c|<here> d|e}| f}| g | h"];
    struct4 [shape=record,label="Class | {<A>A |<B>B| <C>C}"];
    struct5 [shape=record,label="{<D>D |<E>E| <F>F}"];
	struct4:A -> struct5:E [label="foo"];
    struct1:f1 -> struct2:f0;
    struct1:f2 -> struct3:here;
}

 */

public class DottyListener implements ServerListener 
{
	/**
	 * Adds a connection to the server.
	 * @param c the connection to register
	 */
	public void addConnection(Connection c)
	{
		//Debug.trace("Connection added. " + c.toString());
	}
	
	private int m_traces = 0;
	
	private Set m_calls = new TreeSet();
	private Set m_methods = new TreeSet();

	public void addTrace(Connection conn, Trace t)
	{
//		addTraceRecords(conn, t);	
//		addTrace2(conn, t);
		addTraceMethods(conn, t);
		
	}
	
	private Set m_huge = new TreeSet();
	private Vector m_raw = new Vector();
	
	public void addTrace2(Connection conn, Trace t)
	{
		//System.out.println("\n\nsubgraph trace" + ++m_traces +  " { ");
		Vector classes = t.getClasses();
		for (Iterator c = classes.iterator(); c.hasNext(); )
		{
			ClassInfo cli = (ClassInfo)c.next();
			/*
			sb.append("c");
			sb.append(cli.getClassID().toString());
			sb.append(" [shape=record,label=\"");
			sb.append(cli.getName());
			sb.append(" | {");
			*/
			StringBuffer sb = new StringBuffer();			
			sb.append("\"" + cli.getName() + "\" ");
			sb.append(" [label=\"" + cli.getName() + "\"]\n");
			if (shouldInclude(cli.getName()))
				m_methods.add(sb.toString());			
			//Set s = t.getMethodIDsForClass(cli.getClassID());
		}
		
		Vector threads = t.getThreads();
		for (Iterator i = threads.iterator(); i.hasNext(); )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			StringBuffer sh = new StringBuffer();
			if (ti.getFrameCount() > 0)
				sh.append("\"" + t.findClassInfo(ti.getFrameMethod(0)).getName() + "\" ");
			for (int frame = 1; frame < ti.getFrameCount(); frame++)
			{
				if (frame != ti.getFrameCount() - 1)
				sh.append(" -> ");
				MethodIdentifier callee = ti.getFrameMethod(frame);
			sh.append("\"" + t.findClassInfo(callee).getName() + "\"");
			if (frame != ti.getFrameCount() - 1)
				sh.append(" -> ");
			}
			
			if (sh.length() > 0)
			{
				sh.append(";\n");
				m_huge.add(sh.toString());
			}
			
			for (int frame = 0; frame < ti.getFrameCount() - 1; frame++)
			{
				MethodIdentifier callee = ti.getFrameMethod(frame);
				MethodIdentifier caller = ti.getFrameMethod(frame + 1);

			
				String c1 = t.findClassInfo(caller).getName();
				String c2 = t.findClassInfo(callee).getName();
				if (shouldInclude(c1) && shouldInclude(c2))
				{
				StringBuffer sb = new StringBuffer();
				sb.append("\"" + c1 + "\"");
				sb.append(" -> ");
				sb.append("\"" + c2 + "\"");
				sb.append(";\n");
				if (! c1.equals(c2))
				m_calls.add(sb.toString());
				}
			}
		}
	}
	
	public boolean shouldInclude(String className)
	{
	if (1 > 0) return true;
		String[] bad = {"javax.", "java.", "sun."};
		for (int i = 0; i < bad.length; ++i)
			if (className.startsWith(bad[i]))
				return false;
		
		return true;
	}
	
	public void addTraceClasses(Connection conn, Trace t)
	{
		//System.out.println("\n\nsubgraph trace" + ++m_traces +  " { ");
		Vector classes = t.getClasses();
		for (Iterator c = classes.iterator(); c.hasNext(); )
		{
			ClassInfo cli = (ClassInfo)c.next();
			/*
			sb.append("c");
			sb.append(cli.getClassID().toString());
			sb.append(" [shape=record,label=\"");
			sb.append(cli.getName());
			sb.append(" | {");
			*/
			StringBuffer sb = new StringBuffer();			
			sb.append("c" + cli.getClassID());
			sb.append(" [label=\"" + cli.getName() + "\"]\n");
			m_methods.add(sb.toString());			
			Set s = t.getMethodIDsForClass(cli.getClassID());
		}
		
		Vector threads = t.getThreads();
		for (Iterator i = threads.iterator(); i.hasNext(); )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			for (int frame = 0; frame < ti.getFrameCount() - 1; frame++)
			{
				MethodIdentifier callee = ti.getFrameMethod(frame);
				MethodIdentifier caller = ti.getFrameMethod(frame + 1);

				StringBuffer sb = new StringBuffer();
				sb.append("c" + t.findMethodInfo(caller).getClassID().toString());
				sb.append(" -> ");
				sb.append("c" + t.findMethodInfo(callee).getClassID().toString());
				sb.append(";\n");
				m_calls.add(sb.toString());
			}
		}
	}


	public String getFormat(ClassInfo ci, MethodInfo mi)
	{
		String cn = ci.getName();
		String ret = "";
		if (cn.startsWith("java."))
			return ",color=blue,fontcolor=white,style=filled";
		if (cn.startsWith("javax."))
			return ",color=skyblue,fontcolor=white,style=filled";
		if (cn.startsWith("sun."))
			return ",color=purple,fontcolor=white,style=filled";
		else
			return ",color=red,fontcolor=white,style=filled";
		//return "";
	}
	
	/**
	 * Adds a trace for the connection.
	 * @param c the connection
	 * @param t the "latest" trace
	 */
	public void addTraceMethods(Connection conn, Trace t)
	{
		//System.out.println("\n\nsubgraph trace" + ++m_traces +  " { ");
		Vector classes = t.getClasses();
		for (Iterator c = classes.iterator(); c.hasNext(); )
		{
			ClassInfo cli = (ClassInfo)c.next();
			/*
			sb.append("c");
			sb.append(cli.getClassID().toString());
			sb.append(" [shape=record,label=\"");
			sb.append(cli.getName());
			sb.append(" | {");
			*/
			
			Set s = t.getMethodIDsForClass(cli.getClassID());
			for(Iterator method = s.iterator(); method.hasNext(); )
			{
				StringBuffer sb = new StringBuffer();
				MethodInfo mi = t.findMethodInfo( (MethodIdentifier) method.next());
				sb.append("m" + mi.getMethodID());
				sb.append(" [label=\"" + cli.getName() + "\\n" + mi.getFullName() + "\"" + getFormat(cli,mi) + "]");
				m_methods.add(sb.toString());
			}
		}
		
		Vector threads = t.getThreads();
		for (Iterator i = threads.iterator(); i.hasNext(); )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			for (int frame = 0; frame < ti.getFrameCount() - 1; frame++)
			{
				MethodIdentifier callee = ti.getFrameMethod(frame);
				MethodIdentifier caller = ti.getFrameMethod(frame + 1);

				StringBuffer sb = new StringBuffer();
				sb.append("m" + caller.toString());
				sb.append(" -> ");
				sb.append("m" + callee.toString());
				sb.append(" ");
				m_calls.add(sb.toString());
				m_raw.add(sb.toString());
			}
		}
		
		//System.out.println("}\n\n");
	}

	/**
	 * This adds a value to a set contained in map(key); uses HashSet.
	 * @param m a map from key to Set(value)
	 * @param key the key to use when looking through the map
	 * @param value the value to put in the set
	 */
	private static void addItemToSet(Map m, Object key, Object value)
	{
		Assert.notNull(m);
		Assert.notNull(key);
		Assert.notNull(value);

		Set s = (Set) m.get(key);
		if (s == null)
		{
			s = new HashSet();
			m.put(key, s);
		}
		s.add(value);
	}

	private String stripName(String s)
	{
		return s.replace('<', ' ').replace('>', ' ').trim();	
	}
	
	private Map m_recs = new HashMap();
	
	/**
	 * Adds a trace for the connection.
	 * @param c the connection
	 * @param t the "latest" trace
	 */
	public void addTraceRecords(Connection conn, Trace t)
	{
		//System.out.println("\n\nsubgraph trace" + ++m_traces +  " { ");
		Vector classes = t.getClasses();
		for (Iterator c = classes.iterator(); c.hasNext(); )
		{
			ClassInfo cli = (ClassInfo)c.next();
			StringBuffer sb = new StringBuffer();
			sb.append("c");
			sb.append(cli.getClassID().toString());
						
			sb.append(" [shape=record,label=\"");
			sb.append(cli.getName());
			sb.append(" | {");
			
			String cx = "c" + cli.getClassID().toString() + " [shape=record,label=\"" + cli.getName() + " | {";
			
			Set s = t.getMethodIDsForClass(cli.getClassID());
			for(Iterator method = s.iterator(); method.hasNext(); )
			{
				StringBuffer sb1 = new StringBuffer();
				MethodInfo mi = t.findMethodInfo( (MethodIdentifier) method.next());
				sb1.append('<');
				sb1.append("m" + mi.getMethodID());
				sb1.append('>');
				sb1.append(stripName(mi.getFullName())); //stripName
				addItemToSet(m_recs, cx, sb1.toString());
				
				if (method.hasNext()) sb1.append(" | ");				
				sb.append(sb1);
			}
			
			sb.append(" }\"];");
			//System.out.println(sb);
			m_methods.add(sb.toString());
		}
		
		Vector threads = t.getThreads();
		for (Iterator i = threads.iterator(); i.hasNext(); )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			for (int frame = 0; frame < ti.getFrameCount() - 1; frame++)
			{
				MethodIdentifier callee = ti.getFrameMethod(frame);
				MethodIdentifier caller = ti.getFrameMethod(frame + 1);

				StringBuffer sb = new StringBuffer();
				sb.append("c" + t.findMethodInfo(callee).getClassID().toString() + ":m" + callee.toString());
				sb.append(" -> ");
				sb.append("c" + t.findMethodInfo(caller).getClassID().toString() + ":m" + caller.toString() + ";\n");
				//if (!t.findMethodInfo(callee).getClassID().equals(t.findMethodInfo(caller).getClassID()))
				m_calls.add(sb.toString());
			}
		}
		
		//System.out.println("}\n\n");
	}
	
	/**
	 * Removes/unregisters a connection.
	 * @param c the connection to remove
	 */
	public void removeConnection(Connection conn)
	{
		try
		{
			PrintStream out = new PrintStream(new FileOutputStream("dtty.dot"));
			
			out.println("digraph G { ");
			// display the m_calls
			//Collections.sort(m_methods);
			//Collections.sort(m_calls);
			
			for (Iterator c = m_recs.keySet().iterator(); c.hasNext(); )
			{
				String cx = (String)c.next();
				out.print(cx);
				for (Iterator m = ((Set)m_recs.get(cx)).iterator(); m.hasNext(); )
				{
					out.print(m.next());
					if (m.hasNext()) out.print("|");
				}
				out.println("}\"];");
			}
			
			for (Iterator i = new HashSet(m_methods).iterator(); i.hasNext(); )
			{
			out.println(i.next());	
			}
		
			for (Iterator i = new HashSet( m_calls ).iterator(); i.hasNext(); )
			{
				String s = (String)i.next();
				int before = m_raw.size();
				out.print(s);
				m_raw.removeAll( Collections.singleton(s) );
				int after = m_raw.size();
				out.println("[label=\"" + (before - after) + " times\"];");
				//out.println("[style=\"setlinewidth(" + (before - after) + ")\"];");
			}
			
			for (Iterator i = new HashSet(m_huge).iterator(); i.hasNext(); )
				out.println(i.next());
		
			out.println("} ");
			out.close();
			System.err.println("done!");
		}
		catch (Exception e)
		{
			Debug.traceException("removeConn", e);
		}
	}
	
	/**
	 * Sets the port this listener is listening on (for debugging purposes).
	 * @param p port number
	 */
	public void setPort(int p)
	{
		
	}
	
	public static void main(String[] args)
	{
		try
		{
			Server.startServer(Integer.parseInt(args[0]), new DottyListener());
		}
		catch (Exception e)
		{
			Debug.traceException("main", e);
			System.err.println("Usage: DottyListener port");
		}
	}
}

