package org.xdprof.listener;

import org.xdprof.util.*;
import org.xdprof.server.*;
import org.xdprof.data.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class ServerTableGUI extends JFrame implements ServerListener 
{
	/**
	 * The main display panel.
	 */
	private final JPanel m_pane = new JPanel(true);
	private final JPanel m_topPane = new JPanel(new GridLayout(2,1), true);
	private final JPanel m_bottomPane = new JPanel(new GridLayout(1,1), true);
	private static JFrame c_instance;
	public ServerTableGUI()
	{
		super("xdProf Server");
		c_instance = this;
		this.addWindowListener(new WindowAdapter()
			{
			public void windowClosing(WindowEvent e)
			{
				Debug.trace(e.toString());
				System.exit(0);
			}
	    });
		
		m_pane.setLayout(new GridLayout(2,1));
		m_topPane.add(m_lastConn =  new JLabel("x"));
		m_topPane.add(m_lastThread = new JLabel("y"));
		//m_topPane.add(new JLabel("right"), BorderLayout.EAST);
		//m_threadList.setCellRenderer(new ThreadCellRenderer());
		m_pane.add(m_topPane);
		final Font f = new Font("Monospaced", Font.PLAIN, 12);
		
		m_pane.add(m_bottomPane);
        m_pane.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));
		m_pane.setPreferredSize(new Dimension(700,300));
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(m_pane, BorderLayout.CENTER );
		this.getContentPane().add(new JButton("Pause"), BorderLayout.SOUTH);
		this.setVisible(true);
		this.pack();
	}
	private static Vector toVector(Connection c)
	{
		Vector v = new Vector();
		v.add(c.getPID() + "");
		v.add(c.getDescription());
		v.add(c.getInetAddress().toString());
		v.add(c.getVM());
		v.add(c.getOS());
		v.add(c.getPort() + "");
		return v;
	}

	private static Vector toVector(ThreadInfo ti)
	{
		Vector v = new Vector();
		String k = "      " + ti.getStarted();
		v.add(k.substring(k.length() - 5));
		v.add(ti.getThreadID().toString());
		v.add(ti.getName());
		v.add(ti.getParent());
		v.add(ti.getGroup());
		v.add("" + ti.getFrameCount());
		v.add(ti.getThreadStatusDescription());
		return v;
	}
		
	static final Vector c_connectionHeaders = new Vector();
	static final Vector c_threadHeaders = new Vector();
	static final Vector c_stackHeaders = new Vector();
	static
	{
		c_connectionHeaders.add("PID");
		c_connectionHeaders.add("Description");
		c_connectionHeaders.add("Address");
		c_connectionHeaders.add("VM");
		c_connectionHeaders.add("OS");
		c_connectionHeaders.add("Port");
		
		c_threadHeaders.add("Order");
		c_threadHeaders.add("ID");
		c_threadHeaders.add("Name");
		c_threadHeaders.add("Parent");
		c_threadHeaders.add("Group");
		c_threadHeaders.add("# frames");
		c_threadHeaders.add("status");
		
		
		c_stackHeaders.add("Method");
		c_stackHeaders.add("Class");
		c_stackHeaders.add("Line");
	}

	private Component m_lastConn = null;
	private Component  m_lastThread = null;
	private Connection m_selectedConn = null;
	private String m_selectedConnection = null;
	private String m_selectedThread = null;
	
	void updateConnections()
	{
		if (m_lastConn != null)
			m_topPane.remove(m_lastConn);
		
		Vector v = new Vector(m_connections.size());
		int loc = 0, pos = -1;
		for (Iterator i = m_connections.iterator(); i.hasNext(); ++loc )
		{
			Connection c = (Connection)i.next();
			if (c.getDescription().equals(m_selectedConnection))
				pos = loc;
			v.addElement(toVector(c));
		}
		//Debug.trace(v.toString());

		// this uses v and the columnHeaders, and prevents editing
		final TableModel tm = new DefaultTableModel(v, c_connectionHeaders)
			{
				public boolean isCellEditable(int row, int column)
				{
					return false;
				}
			};
	
		final JTable connectionsTable = new JTable(tm);//, c_connectionHeaders, );		
		final ListSelectionModel rowSM = connectionsTable.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener()
			{
			public void valueChanged(ListSelectionEvent e)
			{
			//Ignore extra messages.
			if (e.getValueIsAdjusting()) return;
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
			if (!lsm.isSelectionEmpty())
			{
				int selectedRow = lsm.getMinSelectionIndex();
				//System.out.println("Row " + selectedRow + " is now selected."  + );
				m_selectedConnection = (String)connectionsTable.getModel().getValueAt(selectedRow, 0);
				Connection sel = (Connection)m_connections.get(selectedRow);
				m_bottomPane.removeAll();
				m_selectedConn = sel;
				updateThreads(sel);
				Vector v = (Vector)m_connectionToTraceVector.get(sel);
				Debug.trace(v.lastElement().toString());
			}
			}
			}						   );
		connectionsTable.setSelectionModel(rowSM);
		
		connectionsTable.setCellSelectionEnabled(false);
		connectionsTable.setRowSelectionAllowed(true);
		connectionsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		if (loc != -1 && loc < connectionsTable.getRowCount())
			connectionsTable.setRowSelectionInterval(pos,pos);

		m_topPane.add(m_lastConn = new JScrollPane(connectionsTable), 0);
		
		this.setVisible(true);
		//m_topPane.repaint();
	}

	private synchronized void updateThreads(Connection c)
	{
		if ((c == null) || (!m_connections.contains(c)))
			return;
		Trace t = (Trace)((Vector)m_connectionToTraceVector.get(c)).lastElement();
		Vector v = new Vector();
		int loc = 0;
		int pos = -1;
		for (Iterator i = t.getThreads().iterator(); i.hasNext(); ++loc )
		{
			ThreadInfo ti = (ThreadInfo)i.next();
			if (ti.getThreadID().toString().equals(m_selectedThread))
				pos = loc;
			v.add(toVector(ti));
		}

		synchronized(m_lock)
		{
			if (m_lastThread != null)
				m_topPane.remove(m_lastThread);
		}
		
		final TableModel tm = new DefaultTableModel(v, c_threadHeaders )
			{
				public boolean isCellEditable(int row, int column)
				{
					return false;
				}
			};
	
 		final JTable threadTable = new JTable(tm);
		final ListSelectionModel rowSM = threadTable.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener()
			{
			public void valueChanged(ListSelectionEvent e)
			{
			//Ignore extra messages.
			//if (e.getValueIsAdjusting()) return;
			ListSelectionModel lsm = (ListSelectionModel)e.getSource();
			if (!lsm.isSelectionEmpty())
			{
				int selectedRow = lsm.getMinSelectionIndex();
				synchronized(m_lock)
				{
					m_selectedThread = (String)threadTable.getModel().getValueAt(selectedRow, 1);
				}
				System.out.println("Row " + selectedRow + " is now selected." +	m_selectedThread);
				
				Vector v = (Vector)m_connectionToTraceVector.get(m_selectedConn);
				Trace t = (Trace)v.lastElement();
				ThreadInfo ti = t.getThreadInfo(new ThreadIdentifier(m_selectedThread));
				Vector s = ti.getStackVector();
				Vector q = new Vector();
				for (Iterator i = s.iterator(); i.hasNext(); )
				{
					ThreadInfo.StackFrame sf = (ThreadInfo.StackFrame)i.next();	
					q.add(sf.toVector());
					//Debug.trace(sf.toVector().toString());
				}	
				final JTable m_stackTable = new JTable(q, c_stackHeaders);
		m_stackTable.setDoubleBuffered(true);
				m_bottomPane.removeAll();
				m_bottomPane.add(new JScrollPane(m_stackTable));
				//c_instance.setVisible(true);
				//Connection sel = (Connection)m_connections.get(selectedRow);
				//Ve/ctor v = (Vector)m_connectionToTraceVector.get(sel);
				//Debug.trace(v.lastElement().toString());
			}
			}
			}
		   );
		threadTable.setSelectionModel(rowSM);
		threadTable.setCellSelectionEnabled(false);
		threadTable.setRowSelectionAllowed(true);
		threadTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		if (m_selectedThread != null)
		{
			synchronized(m_lock)
			{
				if (pos != -1 && pos <= threadTable.getRowCount())
					threadTable.setRowSelectionInterval(pos, pos);
			}
		}
		
		
		synchronized(m_lock)
		{
			m_topPane.add(m_lastThread = new JScrollPane(threadTable), -1);
		}
	
		this.setVisible(true);
	}
	
	private Object m_lock = new Object();

	private final Vector m_connections = new Vector();
	private final HashMap m_connectionToTraceVector = new HashMap();
	
	public synchronized void addConnection(Connection c)
	{
		m_connections.add(c);	
		updateConnections();
	}
	
	public synchronized  void addTrace(Connection c, Trace t)
	{
		Debug.trace("Adding trace for " + c);
		//m_connections.add(c);
		Vector v = (Vector)m_connectionToTraceVector.get(c);
		if (v == null)
		{
			v = new Vector();
			m_connectionToTraceVector.put(c, v);
		}
		v.add(t);

		if (m_selectedConn == c)
			updateThreads(c);
		//updateConnections();
	}
	

	
	/**
	 * Removes/unregisters a connection.
	 * @param c the connection to remove
	 */
	public synchronized  void removeConnection(Connection c)
	{
		if (m_selectedConn == c && m_lastConn != null)
		{
			m_topPane.remove(m_lastThread );
			m_topPane.add(m_lastThread = new JLabel(""), -1);
		}
		
		m_connections.remove(c);
		m_connectionToTraceVector.remove(c);
		if (m_connections.size() > 0)
		{
			Connection newSel = (Connection)m_connections.elementAt(0);
			Assert.notNull(newSel);
			m_selectedConn = newSel;
			m_selectedConnection = newSel.getDescription();
			updateConnections();
			updateThreads(newSel);
		}
		else
			updateConnections();
		
	}
	
	/**
	 * Start the program.
	 * @param args args[0] is the port number
	 */
	public static void main(String[] args)
	{
		try
		{
			Server.startServer(Integer.parseInt(args[0]), new ServerTableGUI());
		}
		catch (Exception e)
		{
			Debug.traceException("main", e);
			System.err.println("Usage: ServerGUI port");
		}
	}
	
}
