package org.xdprof.server;

import java.net.InetAddress;
import java.rmi.server.UID;
import java.util.*;
/**
 * This represents a "connection" to a DPROF client
 */
public class Connection
{
	/**
	 * Internal description.
	 */
	private final String m_internal;
	
	private final int m_port;
	
	private final InetAddress m_ia;

	int m_pid;
	String m_description;
	String m_os;
	String m_rt;
	String m_vm;
	HashMap m_properties = new HashMap();

	public int getPID()
	{
		return m_pid;	
	}
	
	public String getOS()
	{
		return m_os;	
	}
	
	public String getRuntime()
	{
		return m_rt;	
	}
	
	public String getVM()
	{
		return m_vm;	
	}
	
	public InetAddress getInetAddress()
	{
		return m_ia;	
	}
	
	public String getDescription()
	{
		return m_description;	
	}
	
	public int getPort()
	{
		return m_port;	
	}
	
	private final String m_s, m_t;
	
	/**
	 * Constructor.
	 * @param s information
	 * @param t more information
	 * @param ia the connection
	 * @param port the port number
	 */
	public Connection(String s, String t, InetAddress ia, int port)
	{
		m_ia = ia;
		m_s = s;
		m_t = t;
		//m_internal = ia.toString() + " " + s + ", port #" +  port;
		m_port = port;
		
		int i = s.indexOf(' ');
		m_pid = Integer.parseInt(s.substring(0, i));
		m_description = s.substring(i + 1, s.indexOf('!',i) - 1);
		StringTokenizer st = new StringTokenizer(s.substring(s.indexOf('!',i)), "!");
		while (st.hasMoreTokens())
		{
			String p = st.nextToken();
			m_properties.put(p.substring(0, p.indexOf('=')), p.substring(p.indexOf('=')+1));
		}
		
//		Debug.trace(m_properties.keySet().toString());
//		Debug.trace(m_properties.values().toString());
		
		m_os = (m_properties.get("os.name") + " " + m_properties.get("os.version") + " " + m_properties.get("os.arch")).trim();
		m_vm = (m_properties.get("java.vm.name") + " " + 
			   m_properties.get("java.vm.version") + " "  + 
			   m_properties.get("java.vm.info") + " "  + 
			   m_properties.get("java.vm.vendor")).trim();
		
		m_rt = (m_properties.get("java.runtime.name") + " "  + 
			   m_properties.get("java.runtime.version")).trim();

		m_internal = ia.toString() + " " + m_description + ", " + m_vm + ", " + m_os + ", port#" + port;
		
		//Debug.trace("os", m_os); Debug.trace("vm", m_vm); Debug.trace("rt", m_rt);
	}
	
	/**
	 * Return a String description.
	 * @return the description
	 */
	public String toString()
	{
		return m_internal;
	}
}
