package org.xdprof.server;

import java.io.*;
import java.net.*;

import org.xdprof.util.*;
import org.xdprof.listener.*;
import org.xdprof.data.*;

/**
 * A Server connects a ServerListener on a port.
 */
public class Server extends Thread
{
	private static Server m_instance = null;
	
	/**
	 * Initialize a Server; must be .start()'ed though.
	 * @param port port number
	 * @param sl the listener
	 */
	private Server(int port, ListenerCollection sl)
	{
		m_port = port;
		m_lc = sl;
		m_running = true;
	}

	public static void startServer(int port, ServerListener sl)
	{
		if (m_instance != null)
			throw new RuntimeException("Server already running!!");
		ListenerCollection lc = new ListenerCollection();
		lc.addServerListener(sl);
		m_instance = new Server(port, lc);
		m_instance.start();
	}
	
	public static void startServer(int port, ListenerCollection lc)
	{
		if (m_instance != null)
			throw new RuntimeException("Server already running!!");

		m_instance = new Server(port, lc);
		m_instance.start();
		
	}
	
	private boolean m_running = false;
	
	/**
	 * The port number.
	 */
	private int m_port;
	
	/**
	 * ServerListener for the connection thread.
	 */
	private ListenerCollection m_lc;
	
	/**
	 * Whether or not we're finished.
	 */
	private boolean m_done = false;

	/*
	public Server(int port, ServerListener sl)
	{
	} // end Server
	*/
	
	/**
	 * Request a shutdown nicely.
	 */
	public void shutdown()
	{
		m_done = true;
	}
	
	/**
	 * Listen on a port and spawn a ConnectionThread for each incoming client.
	 */
	public void run()
	{
		try
		{
			//Debug.trace("Main!");
			ServerSocket ss = new ServerSocket(m_port);
			while (!m_done)
			{
				Socket connection = ss.accept();
				//Debug.trace("Connected to " + ss.getInetAddress().toString());
				ConnectionThread ct = new ConnectionThread(connection, m_lc);
				ct.start();				
			} // end for			
		}
		catch (Exception e)
		{
			Debug.traceException("Server", e);
			System.out.println("Server [port]");
		}
	
	}
}

