package org.xdprof.util;

/**
 * A class for console output and rough profiling.
 */
public final class Debug
{
	/**
	 * Prevent instantiation.
	 */
	private Debug()
	{
	}
	/**
	 * The output for Debug info.
	 */
	private static java.io.PrintStream c_out = System.out;
	/**
	 * The old output destination.
	 */
	private static java.io.PrintStream c_old = null;
	/**
	 * When this was started.
	 */
	private static long c_start = System.currentTimeMillis();
	/**
	 * Whether or not to display thread info.
	 */
	private static boolean c_threads = true;
	
	/**
	 * Display an exception.
	 * @param location where
	 * @param t what
	 */
	public static synchronized void traceException(String location, Throwable t)
	{
		if (c_out != null)
		{
			trace("Exception at " + location);
			t.printStackTrace(c_out);
			c_out.flush();
		}
	}
		
	/**
	 * Turn off debugging.
	 */
	public static synchronized void turnOff()
	{
		c_old = c_out;
		c_out = null;
	}
	
	/**
	 * Turn on debugging.
	 */
	public static synchronized void turnOn()
	{
		c_out = c_old;
		c_old = c_out;
	}
	
	/**
	 * Outputs a string s.
	 * @param s to output
	 */
	public static synchronized void trace(String s)
	{
		if (c_out != null)
		{
			if (c_threads)
			{
				String tname = Thread.currentThread().getName();
				//tname = tname + "                  ";
				//tname = tname.substring(0, 5);
				c_out.print("[" + tname + "] " );
			}
			c_out.print('[');
			c_out.print( (double)((System.currentTimeMillis() - c_start))/(1000.0) );
			c_out.print("s] ");
			c_out.println(s);
			c_out.flush();
		}
	}
	/**
	 * Outputs the variable value.
	 * @param variable variable name
	 * @param o to output
	 */
	public static synchronized void trace(String variable, Object o)
	{
		if (o == null)
			trace(variable + ":\t" + "<null>");
		else
			trace(variable + ":\t" + o.toString());
	}
	
	/**
	 * Outputs the variable value.
	 * @param variable variable name
	 * @param i integer
	 */
	public static synchronized void trace(String variable, int i)
	{
		trace(variable + ":\t" + i);
	}
}