/*
Copyright (c) 2001, John Lambert jlambert@jlambert.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are 
permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of 
conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other
materials provided with the distribution. 

Neither the name of the xdProf project nor the names of its contributors may be used
to endorse or promote products derived from this software without specific prior 
written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
$Header: /xdprof-dll/setup.cpp 7     5/07/01 4:54a Admin $ 
 */
#include "xdprof.h"
#include <jni.h>
// these must be extern-ed in dprof.h
JVMPI_Interface *jvmpi_interface;
JavaVM* GlobalJVM;

inline static string GetSelfInfo()
{
	ostringstream oss;	
	/*
	char compname[MAX_COMPUTERNAME_LENGTH + 1] = {0};
	DWORD size = MAX_COMPUTERNAME_LENGTH;
 	GetComputerName(compname, &size);
	*/
	//oss << "pid = " << 
	oss << _getpid();
	oss << ends;
	string s = oss.str();
	// trim null
	return s.substr(0, s.length() - 1);
}

inline void Leave(string s)
{
	cout << flush << s << endl;
	CALL(ProfilerExit)((jlong)0);
}

//! Returns the program usage.
static string GetUsage()
{
	string usage = "\nXDPROF Usage:\n\n-Xrunxdprof:server=HOSTNAME,port=PORT,refresh=REFRESH[,description=OPTIONAL_DESCRIPTION,hotspot=true]";
	usage = usage + "\n" + "where ";
	usage = usage + "\n" + "\t" + "server=HOSTNAME is the machine running the DPROF server";
	usage = usage + "\n" + "\t" + "port=PORT is the port number of the DPROF server";
	usage = usage + "\n" + "\t" + "refresh=REFRESH is the time to refresh in milliseconds (1000 = 1 sec)";
	usage = usage + "\n" + "" + "Optional arguments: ";
	usage = usage + "\n" + "\t" + "description=OPTIONAL_DESCRIPTION is a string (without spaces) describing this VM";
	usage = usage + "\n" + "\t" + "hotspot=true is set when the HotSpot (non-classic) VM is used (this is not guaranteed to work)";
	usage = usage + "\n" + "\nOther runtime flags that should be passed to the VM:";
	usage = usage + "\n" + "\t" + "-classic -Xdebug -Xnoagent ";
	usage = usage + "\n";
	usage = usage + "\nThe final command line will then look like: \n\n" + 
			"java -classic -Xdebug -Xnoagent -Xrunxdprof:server=localhost,port=1337,refresh=199,description=MyDescription c\n\n";
	return usage;
}


/**
s = "token1=foo,token4=bar,token6=x"
*/
static string getToken(const string &  entire, const string & token )
{
	if (entire.size() == 0 || token.size() == 0)
		return "";
	const string foo = "," + entire + ",";
	string::size_type i = foo.find("," + token + "=");
	if (i == string::npos)
		return "";
	
	int j = i + token.size() + 2;
	int k = foo.find(",", j);
	return foo.substr(j, k - j);
}


// profiler agent entry point
extern "C"
{
#pragma warning(disable:4100)
	JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM *jvm, char *options, void *reserved)
	{
		if ((jvm->GetEnv((void **)&jvmpi_interface, JVMPI_VERSION_1)) < 0)
		{
			fprintf(stderr, "error in obtaining jvmpi interface pointer");
			fflush(stderr);
			return JNI_ERR;
		} 

		GlobalJVM = jvm;
		
		// initialize jvmpi interface
		jvmpi_interface->NotifyEvent = NotifyEvent;

		// initialize monitors
		InitializeMonitors();
		// enable notifications
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_JVM_INIT_DONE               , NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_JVM_SHUT_DOWN				, NULL));

		CHECKEDCALL(EnableEvent(JVMPI_EVENT_THREAD_START, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_THREAD_END, NULL));
		
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_CLASS_LOAD                  , NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_CLASS_UNLOAD                , NULL));

		// This is enabled only temporarily to catch the VM initialization thread
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_OBJECT_ALLOC                  , NULL));

		/*
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_MONITOR_CONTENDED_ENTER		, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_MONITOR_CONTENDED_ENTERED	, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_MONITOR_CONTENDED_EXIT		, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_MONITOR_WAIT				, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_MONITOR_WAITED				, NULL));

		CHECKEDCALL(EnableEvent(JVMPI_EVENT_RAW_MONITOR_CONTENDED_ENTER , NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_RAW_MONITOR_CONTENDED_ENTERED, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_RAW_MONITOR_CONTENDED_EXIT	, NULL));
		CHECKEDCALL(EnableEvent(JVMPI_EVENT_DATA_DUMP_REQUEST			, NULL));
		*/


		
		string selfinfo = GetSelfInfo();
		string hostname = "localhost";
		long port = 1337;
		long refresh = 2000;
		HotSpot = false; // assume we have no HotSpot
		
		if (options != NULL)
		{
			string s(options);
			if (s == "help")
			{
				Leave(GetUsage());
			}
			hostname = getToken(s, "server");
			if (hostname == "")
				Leave("Invalid hostname.");

			port = atoi(getToken(s, "port").c_str());
			if (port <= 0 || port > 65535)
				Leave("Bad port number " + getToken(s, "port"));

			refresh = atoi(getToken(s, "refresh").c_str());
			if (refresh <= 0)
				Leave("Bad refresh number " + getToken(s, "refresh"));

			// add it on
			if (getToken(s, "description") != "")
			{
				selfinfo += ' ';
				selfinfo += getToken(s, "description");
			}
			else
				selfinfo += " none";
			
			if (getToken(s, "hotspot") == "true")
			{
				HotSpot = true;
			}

			debug("All options", s);
		}
		else
		{
			debug("Using defaults...");
		}

		debug("Hostname: ", hostname);
		debug("Port: ", port);
		debug("Refresh: ", refresh);
		debug("SelfInfo: ", selfinfo);
		debug("HotSpot: ", HotSpot ? "true" : "false");

		InitializeBackgroundThread(hostname, port, selfinfo, refresh);

		return JNI_OK;
	}
#pragma warning(default:4100)
}
