/*
Copyright (c) 2001, John Lambert jlambert@jlambert.com
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are 
permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of 
conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice, this list
of conditions and the following disclaimer in the documentation and/or other
materials provided with the distribution. 

Neither the name of the xdProf project nor the names of its contributors may be used
to endorse or promote products derived from this software without specific prior 
written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
$Header: /xdprof-dll/threads.h 22    5/09/01 12:58a Admin $ 
 */
#ifndef _THREADS_H_
#define _THREADS_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "global.h"
#include "types.h"

void ThreadDump(const string& info, ThreadID caller);
extern bool HotSpot;

class ThreadInfo
{
private:
	string m_thread_name;        /* name of thread */
	jobjectID m_thread_id;      /* id of the thread object */
	JNIEnv m_thread_env_id;
	char m_printable[512];
	mutable unsigned long m_started;
public:
	ThreadInfo() { }
	ThreadInfo(const ThreadInfo &);
	ThreadInfo(char *thread_name, char *group_name, char *parent_name, jobjectID thread_id, JNIEnv *thread_env_id);
	inline unsigned long getStarted() const;
	void setStarted(unsigned long started) const;
	inline const char* getShortInfo() const;
	inline string getThreadName() const;
};

class ClassInfo
{
private:
	mutable char* m_printable;
	 char * m_class_name;
	 char * m_source_name;
	jclassID m_class_id;
public:
	ClassInfo(const ClassInfo& );
	~ClassInfo();
	ClassInfo( char * class_name,  char * source_name, jclassID class_id);
	inline const char * getShortInfo() const;
};

class MethodInfo
{
private:
	mutable char * m_short_string;
	jobjectID m_class_id;
	jmethodID m_method_id;
	bool m_used;
	char * m_method_name;
	char * m_method_signature;
public:
	MethodInfo(const MethodInfo &);

	~MethodInfo();

	MethodInfo(
		 char * method_name,
		 char * method_signature,
		jint /* start_lineno */,
		jint /* end_lineno */,
		jmethodID method_id,
		jobjectID class_id);
	
	inline jobjectID getClassID() const;
	const char * getShortInfo() const;
};

#endif // _THREADS_H_

